@@ -68,7 +68,7 @@ class Gutenberg_Dataset():
                 self.cache_dir=None
                 self.log.error(f"Failed to create cache directory {cache_dir}, {e}")
 
-    def _parse_record(self,record,verbose=True):
+    def _parse_record(self, record, verbose=True):
         """ internal function to recreate some consistent record information from near-freestyle text """
         rl=record.split('\n')
         white=str(chr(160))+str(chr(9))+" " # non-breaking space, TAB, and space
@@ -87,7 +87,7 @@ class Gutenberg_Dataset():
         except Exception as e:
             fa=None
             if verbose is True:
-                self.log.warning(f"Failed to apply regex on >{ebook_no}<")
+                self.log.warning(f"Failed to apply regex on >{ebook_no}<: {e}")
             
         if len(rl[0])<5 or fa==None or len(ebook_no)>7:
             if verbose is True:
