@@ -241,7 +241,7 @@ class SelfAttention(layers.Layer):
     that provide context-information for the :math:`input`.
     Input is mutiplied with all three matrices, then :math:`W_k` and :math:`W_q` are multiplied,
     scaled down by :math:`\\sqrt{\\dim{input}[-1]}` and normalized, either by LayerNorm,
-    BatchNorm or Softmax. The result is then multiplied with :math:`W_v`, and, if hidden
+    BatchNorm or Softmax or not at all. The result is then multiplied with :math:`W_v`, and, if hidden
     dimension of the :math:`W_{x_i}` matrices is different from input units last dimension, 
     rescaled by a final dense matrix multiply. Output has same shape as input.
 
@@ -260,11 +260,10 @@ class SelfAttention(layers.Layer):
         #
 
     :param units: Positive integer, number of hidden units. The matrices :math:`W_{x_i}` are of shape :math:`hs \\times hs`.
-    :param norm: either 'batchnorm', 'layernorm, or 'softmax'
+    :param norm: either 'batchnorm', 'layernorm', 'softmax', or None
     """
     def __init__(self, units=None, norm=None, **kwargs):
         super(SelfAttention, self).__init__(**kwargs)
-        self.pm = layers.Permute((2,1))
         self.units = units
         self.norm = norm
         if self.norm=="layernorm":
@@ -273,11 +272,12 @@ class SelfAttention(layers.Layer):
             self.norm = layers.BatchNormalization()
         elif self.norm=="softmax":
             self.norm = layers.Softmax()
+        elif self.norm==None or self.norm == "none":
+            self.norm = None
         else:
-            raise ValueError("Unknown normalization method: {}".format(self.norm))
+            raise ValueError("Unknown norm: {}".format(self.norm))
 
     def build(self, input_shape):
-        # super(SelfAttention, self).build(input_shape)
         self.fact = math.sqrt(input_shape[-1])
         if self.units is None:
             dim2 = input_shape[-1]
@@ -301,18 +301,19 @@ class SelfAttention(layers.Layer):
         return config 
 
     def call(self, inputs):
-        # ip = self.pm(inputs)
         vk = tf.matmul(inputs, self.w_keys)
         vq = tf.matmul(inputs, self.w_queries)
         vv = tf.matmul(inputs, self.w_values)
         kq = tf.matmul(vk, vq, transpose_b=True)
         kqs = kq/self.fact
-        sn = self.norm(kqs)
-        # print(f"sm={sm.shape}, vv={vv.shape}")
+        if self.norm is not None:
+            sn = self.norm(kqs)
+        else:
+            sn = kqs
         out = tf.matmul(sn, self.pm(vv), transpose_b=True)
+
         if self.units is not None:
             out = tf.matmul(out, self.scale)
-        # out = self.pm(out)
         return out
 
 class MultiHeadSelfAttention(layers.Layer):
@@ -428,3 +429,37 @@ class MultiHeadSelfAttention(layers.Layer):
         if self.final_relu is True:
             x = self.relu2(x)
         return x
+
+class PositionalEncoding(layers.Layer):
+    """ Positional encoding layer.
+
+    adds sinusoid of different frequencies to the input.
+    """
+    def __init__(self, **kwargs):
+        super(PositionalEncoding, self).__init__(**kwargs)
+
+    # positional encoding taken from: https://www.tensorflow.org/text/tutorials/transformer
+    @staticmethod
+    def _get_angles(pos, i, d_model):
+        angle_rates = 1 / np.power(10000, (2 * (i//2)) / np.float32(d_model))
+        return pos * angle_rates
+
+    @staticmethod
+    def _positional_encoding(position, d_model):
+        angle_rads = MultiHeadSelfAttention._get_angles(np.arange(position)[:, np.newaxis],
+                                                        np.arange(d_model)[np.newaxis, :],
+                                                        d_model)
+        # apply sin to even indices in the array; 2i
+        angle_rads[:, 0::2] = np.sin(angle_rads[:, 0::2])
+        # apply cos to odd indices in the array; 2i+1
+        angle_rads[:, 1::2] = np.cos(angle_rads[:, 1::2])
+        pos_encoding = angle_rads[np.newaxis, ...]
+        return tf.cast(pos_encoding, dtype=tf.float32)
+
+
+    def build(self, input_shape):
+        self.pe = self._positional_encoding(input_shape[1], input_shape[2])
+
+    def call(self, inputs):
+        return tf.add(inputs, self.pe)
+
